
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "polygraph.h"

#include <iostream.h>
#include <iomanip.h>

#include "OLog.h"
#include "ILog.h"
#include "ObjId.h"
#include "Oid.h"

OidRec::OidRec(const ObjId &id) {
	reset();
	wid = id.world();
	hidx = id.server();
	oid = id.name();
	tid = id.type();
}

void OidRec::reset() {
	wid.clear();
	tid = oid = hidx = -1;
	theFlags = 0;
}

bool OidRec::direct() const {
	return !embedded();
}

bool OidRec::embedded() const {
	Assert(wid);
	return wid.mutant();
}

// /*first*/ cachable miss
bool OidRec::fill() const {
	return basic() && /* !repeat() &&*/ cachable() && !hit();
}

ostream &OidRec::printOid(ostream &os) const {
	const ios::fmtflags flags = os.flags();
	const char fill = os.fill('0');
	os << hex << setfill('0') << setw(8) << oid;
	os.fill(fill);
	os.flags(flags);
	return os;
}

ostream &OidRec::printUriPath(ostream &os) const {
	// XXX: merge with Oid2Url somewhow
	os << 'w' << wid << '/';
	if (tid) {
		const ios::fmtflags flags = os.flags();
        const char fill = os.fill('0');
        os << hex << setfill('0') << 't' << setw(2) << tid << '/';
        os.fill(fill);
        os.flags(flags);
	}
	printOid(os << '_');
	return os;
}

ostream &OidRec::printFlags(ostream &os) const {
	if (repeat())
		os << "rpt,";
	if (hot())
		os << "hot,";
	if (foreign())
		os << "foreign,";

	if (basic())
		os << "basic,";
	if (ims())
		os << "ims,";
	if (reload())
		os << "reload,";

	if (cachable())
		os << "chb,";
	if (hit())
		os << "hit";

	return os;
}

ostream &OidRec::print(ostream &os) const {
	os << "srv: " << hidx;
	printUriPath(os << " path: ");
	printFlags(os << " flags: ");
	return os;
}

void OidRec::store(OLog &ol) const {
	ol << hidx << tid << oid << (int)theFlags;
}

void OidRec::load(ILog &il) {
	int f = -1;
	il >> hidx >> tid >> oid >> f;
	theFlags = (unsigned)f;
}
