
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_OID_RECORD_CLASS_H
#define PG_OID_RECORD_CLASS_H

#include "base/UniqId.h"

class ObjId;

// storage for oid properties
class OidRec {
	protected:
		typedef enum { flgNone = 0, flgRepeat = 1, flgForeign = 2,
			flgCachable = 4, flgHit = 8, flgIms = 16, flgReload = 32,
			flgHot = 64 } Flag;

	public:
		OidRec() { reset(); }
		OidRec(const ObjId &id);

		void reset();

		bool direct() const;
		bool embedded() const;
		bool repeat() const { return theFlags & flgRepeat; }
		bool hot() const { return theFlags & flgHot; }
		bool foreign() const { return theFlags & flgForeign; }

		bool basic() const { return !(ims() || reload()); }
		bool ims() const { return theFlags & flgIms; }
		bool reload() const { return theFlags &  flgReload; }
		
		bool offeredHit() const { return basic() && repeat() && cachable(); }
		bool hit() const { return theFlags & flgHit; }
		bool cachable() const { return theFlags & flgCachable; }
		bool fill() const;

		void repeat(bool be) { setFlag(flgRepeat, be); }
		void hot(bool be) { setFlag(flgHot, be); }
		void foreign(bool be) { setFlag(flgForeign, be); }

		void ims(bool be) { setFlag(flgIms, be); }
		void reload(bool be) { setFlag(flgReload, be); }
		
		void hit(bool be) { setFlag(flgHit, be); }
		void cachable(bool be) { setFlag(flgCachable, be); }

		ostream &print(ostream &os) const;
		ostream &printOid(ostream &os) const;
		ostream &printUriPath(ostream &os) const;
		ostream &printFlags(ostream &os) const;

		void store(OLog &ol) const;
		void load(ILog &il);

	protected:
		inline void setFlag(int flag, bool set);

	public:
		UniqId wid;     // server world id (cached)
		int hidx;	// host index
		int tid;        // object content type id
		int oid;        // object id

	protected:
		unsigned int theFlags;
};


inline
ostream &operator <<(ostream &os, const OidRec &r) {
	return r.printOid(os);
}

inline
OLog &operator <<(OLog &ol, const OidRec &r) {
	r.store(ol);
	return ol;
}

inline
ILog &operator >>(ILog &il, OidRec &r) {
	r.load(il);
	return il;
}

inline
void OidRec::setFlag(int flag, bool set) {
	if (set)
		theFlags |= flag;
	else
		theFlags &= ~flag;
}

#endif
