
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "base/polygraph.h"

#include "base/ILog.h"
#include "base/OLog.h"
#include "base/OidGenStat.h"


OidGenStat::OidGenStat() {
	reset();
}

void OidGenStat::reset() {
	memset(theNeed, 0, sizeof(theNeed));
	memset(theLack, 0, sizeof(theLack));
	memset(theGen, 0, sizeof(theGen));
	waitForGen = false;
}

void OidGenStat::recordNeed(bool rep, bool pub) {
	Assert(!waitForGen);
	needRep = rep;
	needPub = pub;
	waitForGen = true;
}

void OidGenStat::recordGen(bool rep, bool pub) {
	Assert(waitForGen);
	theNeed[toIdx(needRep, needPub)]++;
	if (needRep != rep || needPub != pub)
		theLack[toIdx(needRep, needPub)]++;
	theGen[toIdx(rep, pub)]++;
	waitForGen = false;
}

OidGenStat &OidGenStat::operator +=(const OidGenStat &s) {
	mergeTable(theNeed, s.theNeed);
	mergeTable(theLack, s.theLack);
	mergeTable(theGen, s.theGen);
	return *this;
}

void OidGenStat::store(OLog &log) const {
	writeTable(log, theNeed);
	writeTable(log, theLack);
	writeTable(log, theGen);
}

void OidGenStat::load(ILog &log) {
	readTable(log, theNeed);
	readTable(log, theLack);
	readTable(log, theGen);
}

ostream &OidGenStat::print(ostream &os, const String &pfx) const {
	os << pfx << "categories.count:\t " << 3 << endl;
	printTable(os, pfx + "need.", theNeed);
	printTable(os, pfx + "lack.", theLack);
	printTable(os, pfx + "gen", theGen);
	return os;
}

void OidGenStat::mergeTable(Table &a, const Table &b) const {
	for (int i = 0; i < 4; ++i)
		a[i] += b[i];
}

void OidGenStat::writeTable(OLog &log, const Table &t) const {
	for (int i = 0; i < 4; ++i)
		log << t[i];
}

void OidGenStat::readTable(ILog &log, Table &t) {
	for (int i = 0; i < 4; ++i)
		log >> t[i];
}

void OidGenStat::printTable(ostream &os, const String &pfx, const Table &t) const {
	os << pfx << "new_prv.count:\t " << t[toIdx(false, false)] << endl;
	os << pfx << "new_pub.count:\t " << t[toIdx(false, true)] << endl;
	os << pfx << "rep_prv.count:\t " << t[toIdx(true, false)] << endl;
	os << pfx << "rep_pub.count:\t " << t[toIdx(true, true)] << endl;
}

