
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */


#include "polygraph.h"

#include "base/ContTypeStat.h"
#include "runtime/ErrorMgr.h"
#include "csm/ContentCfg.h"
#include "csm/ContentMgr.h"

#include "pgl/ContentSym.h"

#include "runtime/polyErrors.h"

ContentMgr TheContentMgr;


ContentMgr::ContentMgr() {
}

ContentMgr::~ContentMgr() {
	while (theCfgs.count()) delete theCfgs.pop();
	while (theSyms.count()) delete theSyms.pop();
}

// this method can be called re-entrantly!
ContentCfg *ContentMgr::get(const ContentSym *cs) {
	Assert(cs);

	int idx;
	for (idx = 0; idx < theSyms.count(); ++idx) {
		if (cs->equal(*theSyms[idx]))
			return theCfgs[idx];
	}

	// no match found
	ContentCfg *cfg = new ContentCfg(idx+1);
	theCfgs.append(cfg);
	theSyms.append((ContentSym*)cs->dup());

	// configure after append to facilitate reentrant calls
	cfg->configure(cs);
	ContTypeStat::RecordKind(cfg->id(), cfg->kind());
	return cfg;
}

bool ContentMgr::validId(int id) const {
	const int idx = id - 1;
	return 0 <= idx && idx < theCfgs.count();
}

ContentCfg *ContentMgr::get(int id) {
	const int idx = id - 1;
	Assert(0 <= idx && idx < theCfgs.count());
	return theCfgs[idx];
}

void ContentMgr::get(const Array<ContentSym*> &syms, Array<ContentCfg*> &cfgs) {
	for (int i = 0; i < syms.count(); ++i)
		cfgs.append(get(syms[i]));
}
