
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "pgl.h"

#include "String.h"
#include "PglRec.h"
#include "PglStringSym.h"
#include "PglArraySym.h"
#include "MimeSym.h"


MimeSym::MimeSym(): RecSym("Mime", new PglRec) {
	theRec->bAdd("string", "type", 0);
	theRec->bAdd("string[]", "extensions", 0);
}

MimeSym::MimeSym(const String &aType, PglRec *aRec): RecSym(aType, aRec) {
}

bool MimeSym::isA(const String &type) const {
	return RecSym::isA(type) || type == "Mime";
}

SynSym *MimeSym::clone(const String &type) const {
	if (isA(type))
		return new MimeSym(this->type(), theRec->clone());
	return RecSym::clone(type);
}

String MimeSym::mimeType() const {
	SynSymTblItem *ti = 0;
	Assert(theRec);
	Assert(theRec->find("type", ti));
	return ti->sym() ?
		((const StringSym&)ti->sym()->cast("string")).val() : String(0);
}

bool MimeSym::extensions(Array<String*> &exts, RndDistr *&selector) const {
	SynSymTblItem *ei = 0;
	Assert(theRec->find("extensions", ei));
	if (!ei->sym())
		return false; // undefined

	// build extensions array and selector
	ArraySym &es = (ArraySym&)ei->sym()->cast("Array");
	selector = es.makeSelector("mime-extensions");
	for (int i = 0; i < es.count(); ++i) {
		const String &ext = ((const StringSym&)es[i]->cast("string")).val();
		exts.append(new String(ext));
	}

	return true;
}
