
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "pgl.h"

#include "Assert.h"
#include "String.h"
#include "Rnd.h"
#include "PglDistrSym.h"
#include "PglQualifSym.h"


QualifSym::QualifSym(Kind aKind): ExpressionSym("qualifier"), theKind(aKind) {
}

bool QualifSym::isA(const String &type) const {
	return ExpressionSym::isA(type) || type == "Qualif";
}

SynSym *QualifSym::clone(const String &type) const {
	if (isA(type))
		return new QualifSym(theKind);
	return ExpressionSym::clone(type);
}

// lmt + expr  or  now + expr
ExpressionSym *QualifSym::bnOper(const String &op, const SynSym &exp) const {
	if (op != "+")
		return ExpressionSym::bnOper(op, exp);

	Assert(theKind != qfNone);
	DistrSym *d = (DistrSym*)exp.clone("time_distr");
	if (!d)
		cerr << exp.loc() << "`time_distr' expected after a qualifier; got `" << exp.type() << "'" << endl << xexit;
	d->qualifier(this, op);
	return d;
}

ostream &QualifSym::print(ostream &os, const String &) const {
	if (theKind == qfLmt)
		os << "lmt";
	else
	if (theKind == qfNow)
		os << "now";
	else
	if (theKind == qfNmt)
		os << "nmt";
	return os;
}
