
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "polygraph.h"

#include "runtime/Connection.h"

#include "base/ObjId.h"
#include "Server.h"
#include "Cache.h"
#include "DistrPoint.h"
#include "runtime/HostMap.h"
#include "PxySrvXact.h"

#include "runtime/polyErrors.h"
#include "runtime/globals.h"


void PxySrvXact::reset() {
	CacheReader::reset();
	SrvXact::reset();
	waitForCache = true;
}

// also called from noteCacheReady
void PxySrvXact::noteWriteReady(int fd) {
	Assert(theDistrPoint);

	const Size prodObjSize = theProducedSize - theRepHdrSize;
	const Size objSize = theDistrPoint->entry()->objSize();
	const Size pendSize = theDistrPoint->readySize() <= 0 ?
		Size(0) : theDistrPoint->readySize() - prodObjSize;

	if (objSize >= 0) {
		Assert(prodObjSize < objSize);
		if (theRepSize >= 0)
			Assert(objSize == theRepSize - theRepHdrSize);
	}

	waitForCache = pendSize <= 0;

	if (0 && waitForCache) { //  wasWFC != waitForCache) {
		// XXX: remove debugging
		const Size diff1 = theProducedSize-theRepHdrSize;
		const Size diff2 = diff1 - theDistrPoint->readySize();
		cerr << here 
			// << theDistrPoint << ": changed wait: " << wasWFC << " to " << waitForCache << " ( "
			<< theProducedSize << " - " << theRepHdrSize << " < " << theDistrPoint->readySize() << " < " << theRepSize
			<< "; diffs: " << diff1 << "/" << diff2 << endl;
	}

	if (waitForCache) {
		// writer may not exist yet
		// XXX: we need to set a timeout if waiting for the writer
		// Assert(theDistrPoint->writer());
		Assert(prodObjSize < objSize || objSize < 0);
		theConn->theWr.stop(this);
	} else {
		Assert(pendSize > 0);
		theConn->maxIoSize(pendSize);
		SrvXact::noteWriteReady(fd);
	}
}

// XXX: nobody calls these?
void PxySrvXact::noteWriterLeft() {
	finish(errPrematureEof);
}

void PxySrvXact::noteCacheReady() {
	if (waitForCache)
		SrvXact::noteWriteReady(theConn->fd()); 
}

void PxySrvXact::newState(State aState) {
	SrvXact::newState(aState);
	if (theState == stSpaceWaiting) {
		Assert(!theDistrPoint);

		// XXX: cheat so that our clients know it is a hit
		if (theOwner->cache()->cached(theOidRec)) {
			theReqHdr.theXactId = theId;
		} else {
			Assert(theReqHdr.theXactId != theId);
		}
		Assert(theReqHdr.theXactId);

		theDistrPoint = theOwner->cache()->addReader(theOidRec, this);
		if (!theDistrPoint) {
			// the cache must report an error
			// XXX: newState(stSpaceWaiting) may not be the last call in chain!
			finish(-1);
			return;
		}
	} else
	if (theState == stDone) {
		if (theDistrPoint) {
			theDistrPoint->delReader(this); // XXX: check that it resets theDistrPoint!
			Assert(!theDistrPoint);
		}
	}
}
