
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_CONNECTION_CLASS_H
#define PG_CONNECTION_CLASS_H

#include "xstd/Queue.h"
#include "xstd/NetAddr.h"
#include "xstd/Socket.h"
#include "xstd/FileScanner.h"
#include "base/ConnCloseStat.h"
#include "runtime/IOBuf.h"

class ConnMgr;
class PortMgr;

// bi-directional connection based on a socket with I/O buffers

class Connection: public QueueItem {
	public:
		typedef ConnCloseStat::CloseKind CloseKind;

		class HalfPipe {
			public:
				HalfPipe(Connection &aConn, IOBuf &aBuf, IODir aDir):
					theConn(aConn), theBuf(aBuf), theDir(aDir) { reset(); }

				void reset();
				void start(FileScanUser *u);
				void start(FileScanUser *u, Time timeout);
				void stop(FileScanUser *u);

			public:
				Connection &theConn;
				IOBuf &theBuf;
				IODir theDir;
				FileScanReserv theReserv;
				int theIOCnt;
				bool isReady;
		};
		friend class HalfPipe;

	public:
		Connection();
		~Connection() { if (theSock) close(); }

		void reset();

		int seqId() const { return theSeqId; }
		Socket &sock() { return theSock; }
		int fd() const { return theSock.fd(); }
		const NetAddr &addr() const { return theAddr; }
		int rport() const;
		ConnMgr *mgr() const { return theMgr; }
		int logCat() const { return theLogCat; }

		bool exhausted() const { return isAtEof && !theRdBuf.contSize(); }
		bool atEof() const { return isAtEof; }
		bool bad() const { return isBad; }
		CloseKind closeKind() const { return theCloseKind; }

		bool connect(const NetAddr &addr, const SockOpt &opt, PortMgr *aPortMgr);
		bool accept(Socket &s, const SockOpt &opt, bool &fatal);
		Size read();
		Size write();
		bool close();

		void mgr(ConnMgr *aMgr) { theMgr = aMgr; }
		void logCat(int aLogCat) { theLogCat = aLogCat; }
		
		void useLimit(int aLimit) { theUseLmt = aLimit; }
		bool reusable() const;
		int useCnt() const { return theUseCnt; }
		Time useStart() const { return theUseStart; }
		void use();

		Time openTime() const { return theOpenTime; }
		int ioCnt() const { return theRd.theIOCnt + theWr.theIOCnt; }

		void bad(bool be) { isBad = be; }
		void lastUse(bool be) { isLastUse = be; }
		void closeKind(CloseKind aKind) { theCloseKind = aKind; }

		void maxIoSize(Size aMax) { theMaxIoSize = aMax; }

	protected:
		bool setSockOpt(const SockOpt &opt);
		bool preIO(HalfPipe &ioPipe);

	public:
		RdBuf theRdBuf;
		WrBuf theWrBuf;

		HalfPipe theRd; // these refer to bufs; must go after them!
		HalfPipe theWr;

	protected:
		Socket theSock;
		NetAddr theAddr;
		mutable ConnMgr *theMgr;
		PortMgr *thePortMgr;    // used to pass address to bind call

		int theLocPort;
		mutable int theRemPort; // used as a cache for sock().rport()

		Time theOpenTime;
		Time theUseStart;       // start of the current/last "use"
		int theUseLmt;          // maximum number of uses allowed
		int theUseCnt;          // number of "uses" since open()
		CloseKind theCloseKind; // how the connection was closed

		Size theMaxIoSize;      // per-io limit, reset after each IO

		int theLogCat;          // log entry category

		bool isBad;
		bool isAtEof;
		bool isLastUse;         // this must be the last use of the conn

	private:
		static int TheLastSeqId;
		int theSeqId;
};

#endif
