
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "base/polygraph.h"

#include "base/UniqId.h"
#include "runtime/StatPhase.h"
#include "runtime/StatPhaseMgr.h"
#include "server/CltGrpGlobMap.h"

class CltGrpGlobRep {
	public:
		CltGrpGlobRep(const UniqId &aGroup): theGroup(aGroup), thePhaseSyncPos(-1) {}

		const UniqId &group() const { return theGroup; }
		int phaseSyncPos() const { return thePhaseSyncPos; }

		void phaseSyncPos(int aPos) { thePhaseSyncPos = aPos; }

	protected:
		UniqId theGroup;
		int thePhaseSyncPos;
};


CltGrpGlobMap TheCltGrpGlobMap;


// XXX: we need to grow this map on-demand basis!
CltGrpGlobMap::CltGrpGlobMap(): theIndex(11), theCount(0), thePhaseSyncPosMin(-1) {
}

CltGrpGlobMap::~CltGrpGlobMap() {
	while (theIndex.count())
		delete theIndex.pop();
}

void CltGrpGlobMap::notePhaseSync(const UniqId &group, int pos) {
	int idx = -1;
	if (!find(group, idx))
		addAt(idx, group);
	CltGrpGlobRep *rep = theIndex[idx];

	const int oldPos = rep->phaseSyncPos();
	if (oldPos == pos)
		return;
	rep->phaseSyncPos(pos);
	
	if (oldPos <= thePhaseSyncPosMin) {
		thePhaseSyncPosMin = -1;
		bool inited = false;
		for (int i = 0; i < capacity(); ++i) {
			if (CltGrpGlobRep *r = theIndex[i]) {
				if (!inited || r->phaseSyncPos() < thePhaseSyncPosMin) {
					thePhaseSyncPosMin = r->phaseSyncPos();
					inited = true;
				}
			}
		}

		if (TheStatPhaseMgr->unlockToStop() && TheStatPhaseMgr.phaseSyncPos() <= thePhaseSyncPosMin)
			TheStatPhaseMgr->unlock();
	}
}

bool CltGrpGlobMap::find(const UniqId &group, int &idx) const {
	if (!capacity())
		return false;

	// start with a hash and then try linear search
	idx = group.hash() % capacity();
	for (int i = capacity(); i; --i) {
		bool res = false;
		if (endSearch(group, idx, res))
			return res;
		idx++;
		idx %= capacity();
	}

	Assert(false); // no empty slots left!
	return false;
}

CltGrpGlobRep *CltGrpGlobMap::addAt(int idx, const UniqId &group) {
	Assert(group);
	Assert(!theIndex[idx]);
	theIndex.put(new CltGrpGlobRep(group), idx);
	theCount++;
	return theIndex[idx];
}

// returns true if there is no reason to search further (match or empty)
bool CltGrpGlobMap::endSearch(const UniqId &group, int idx, bool &res) const {
	if (CltGrpGlobRep *h = theIndex[idx]) {
		if (h->group() == group)
			return res = true;
		return res = false;
	}

	// found empty slot
	res = false;
	return true;
}
