
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "xparser.h"

#include <iostream.h>
#include <strstream.h>

#include "SynSym.h"


/* SynSym */

SynSym::SynSym(const String &aType): theType(0) {
	type(aType);
}

SynSym::~SynSym() {
	type(0);
}

bool SynSym::isA(const String &type) const {
	return theType && type && !strcmp(type, theType);
}

// may be expensive
bool SynSym::canBe(const String &type) const {
	if (isA(type))
		return true;
	if (SynSym *s = clone(type)) {
		delete s;
		return true;
	}
	return false;
}

// expensive: comparison is done using print method!
bool SynSym::equal(const SynSym &s) const {
	ostrstream os1, os2;
	print(os1) << ends;
	s.print(os2) << ends;
	
	const bool res = strcmp(os1.str(), os2.str()) == 0;

	os1.freeze(0);
	os2.freeze(0);

	return res;
}

SynSym &SynSym::cast(const String &typ) {
	((const SynSym *)this)->cast(typ); // will abort if needed
	return *this;
}

const SynSym &SynSym::cast(const String &typ) const {
	if (!isA(typ))
		cerr << here << "incompatible symbol types: "
			<< type() << " ? " << typ << endl << xabort;
	return *this;
}

SynSym *SynSym::clone(const String &) const {
	return 0;
}

void SynSym::type(const String &aType) {
	theType = aType;
}

ostream &SynSym::print(ostream &os, const String &) const {
	return os << "<something " << type() << "-ish>";
}
