
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "xstd.h"

#include <sys/types.h>
#include <sys/socket.h>
#include <net/if.h>
#include <arpa/inet.h>

#include <iostream.h>

#include "Assert.h"
#include "getIfAddrs.h"
#include "gadgets.h"

static int ifreqSize(const ifreq *req);

class IfConf: public ifconf {
	public:
		IfConf(int len = 0) { ifc_buf = 0; setLen(len); }
		~IfConf() { setLen(0); }

		void setLen(int len);

		int reqSize(const struct ifreq *req) const;
		const ifreq *req(int off) const { return (const ifreq *)(ifc_buf + off); }
		operator void*() const { return (void*)ifc_buf; }
};

void IfConf::setLen(int len) { 
	delete[] ifc_buf;
	ifc_buf = len > 0 ? new char[len] : 0;
	ifc_len = len;
}

int IfConf::reqSize(const struct ifreq *req) const {
	// note: there are many broken cases not covered here;
	// see what /usr/src/contrib/bind/bin/named/ns_main.c has to do
#if defined(HAVE_SA_LEN)
	const int varLen = Max((int)sizeof(sockaddr), (int)req->ifr_addr.sa_len);
#else
	const int varLen = (int)sizeof(sockaddr);
#endif
	return sizeof(req->ifr_name) + varLen;
}

// guess how big the cfg buffer should be
// should this be a Socket method?
static
bool GuessIfConfSize(Socket s, IfConf &cfg) {
	int last_returned_len = -1;
	// grow the buffer until we get errors or the answer fits
	// the asnwer fits if returned len does not change
	for (int buf_len = 4*1024; s; buf_len *= 2) {
		cfg.setLen(buf_len);
		if (!s.getIfConf(cfg))
			return false;
		if (cfg.ifc_len == last_returned_len)
			return true;
		last_returned_len = cfg.ifc_len;
	}
	return false;
}

static
bool ifNameMatch(const char *ifname, const char *aname) {
	if (const char *p = strchr(aname, ':'))
		return strncmp(ifname, aname, p - aname - 1) == 0;
	else
		return strcmp(ifname, aname) == 0;
}

// Get a list of all interface addresses using Socket::getIfConf.  
bool GetIfAddrs(Array<in_addr> &addrs, const char *ifname) {
	Socket s;
	if (!s.create(AF_INET, SOCK_DGRAM, 0))
		return false;

	IfConf cfg;
	if (GuessIfConfSize(s, cfg)) {
		const ifreq *req;
		for (int off = 0; off < cfg.ifc_len; off += ifreqSize(req)) {
			req = cfg.req(off);

			if (req->ifr_addr.sa_family != AF_INET)
				continue;

			if (ifname && !ifNameMatch(ifname, req->ifr_name))
				continue;

			addrs.append(((const sockaddr_in &)req->ifr_addr).sin_addr);
		}
	}

	s.close();
	return addrs.count();
}


// calculate the actual size of ifreq structure
static
int ifreqSize(const ifreq *req) {
	// note: there are many broken cases not covered here;
	// see what /usr/src/contrib/bind/bin/named/ns_main.c has to do

#if defined HAVE_SA_LEN
	return Max(sizeof(*req), req->ifr_addr.sa_len + sizeof(req->ifr_name));
#else
	return sizeof(*req);
#endif
}
